@@ -28,20 +28,29 @@ from pyoverkiz.const import (
     SUPPORTED_SERVERS,
 )
 from pyoverkiz.exceptions import (
+    AccessDeniedToGatewayException,
     BadCredentialsException,
     CozyTouchBadCredentialsException,
     CozyTouchServiceException,
     InvalidCommandException,
     InvalidEventListenerIdException,
+    InvalidTokenException,
     MaintenanceException,
+    MissingAuthorizationTokenException,
     NexityBadCredentialsException,
     NexityServiceException,
     NoRegisteredEventListenerException,
     NotAuthenticatedException,
+    NotSuchTokenException,
+    SessionAndBearerInSameRequestException,
     SomfyBadCredentialsException,
     SomfyServiceException,
+    TooManyAttemptsBannedException,
+    TooManyConcurrentRequestsException,
     TooManyExecutionsException,
     TooManyRequestsException,
+    UnknownObjectException,
+    UnknownUserException,
 )
 from pyoverkiz.models import (
     Command,
@@ -50,6 +59,7 @@ from pyoverkiz.models import (
     Execution,
     Gateway,
     HistoryExecution,
+    LocalToken,
     OverkizServer,
     Place,
     Scenario,
@@ -68,7 +78,7 @@ async def refresh_listener(invocation: dict[str, Any]) -> None:
     await invocation["args"][0].register_event_listener()
 
 
-# pylint: disable=too-many-instance-attributes
+# pylint: disable=too-many-instance-attributes, too-many-branches
 
 
 class OverkizClient:
@@ -92,6 +102,7 @@ class OverkizClient:
         username: str,
         password: str,
         server: OverkizServer,
+        token: str | None = None,
         session: ClientSession | None = None,
     ) -> None:
         """
@@ -106,6 +117,7 @@ class OverkizClient:
         self.username = username
         self.password = password
         self.server = server
+        self._access_token = token
 
         self.setup: Setup | None = None
         self.devices: list[Device] = []
@@ -140,6 +152,16 @@ class OverkizClient:
         Authenticate and create an API session allowing access to the other operations.
         Caller must provide one of [userId+userPassword, userId+ssoToken, accessToken, jwt]
         """
+        # Local authentication
+        if "/enduser-mobile-web/1/enduserAPI/" in self.server.endpoint:
+            if register_event_listener:
+                await self.register_event_listener()
+            else:
+                # Call a simple endpoint to verify if our token is correct
+                await self.get_gateways()
+
+            return True
+
         # Somfy TaHoma authentication using access_token
         if self.server == SUPPORTED_SERVERS["somfy_europe"]:
             await self.somfy_tahoma_get_access_token()
@@ -210,10 +232,16 @@ class OverkizClient:
 
             return self._access_token
 
-    async def somfy_tahoma_refresh_token(self, refresh_token: str) -> str:
+    async def refresh_token(self) -> None:
         """
-        To get a new access token, the refresh token is needed. The refresh_token will be valid 14 days.
+        Update the access and the refresh token. The refresh token will be valid 14 days.
         """
+        if self.server != SUPPORTED_SERVERS["somfy_europe"]:
+            return
+
+        if not self._refresh_token:
+            raise ValueError("No refresh token provided. Login method must be used.")
+
         # &grant_type=refresh_token&refresh_token=REFRESH_TOKEN
         # Request access token
         async with self.session.post(
@@ -221,7 +249,7 @@ class OverkizClient:
             data=FormData(
                 {
                     "grant_type": "refresh_token",
-                    "refresh_token": refresh_token,
+                    "refresh_token": self._refresh_token,
                     "client_id": SOMFY_CLIENT_ID,
                     "client_secret": SOMFY_CLIENT_SECRET,
                 }
@@ -244,8 +272,6 @@ class OverkizClient:
                 seconds=token["expires_in"] - 5
             )
 
-            return self._access_token
-
     async def cozytouch_login(self) -> str:
         """
         Authenticate via CozyTouch identity and acquire JWT token.
@@ -487,6 +513,7 @@ class OverkizClient:
         """
         await self.__post("setup/devices/states/refresh")
 
+    @backoff.on_exception(backoff.expo, TooManyConcurrentRequestsException, max_tries=5)
     async def register_event_listener(self) -> str:
         """
         Register a new setup event listener on the current session and return a new
@@ -503,6 +530,7 @@ class OverkizClient:
 
         return listener_id
 
+    @backoff.on_exception(backoff.expo, TooManyConcurrentRequestsException, max_tries=5)
     @backoff.on_exception(
         backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
     )
@@ -519,6 +547,7 @@ class OverkizClient:
         Per-session rate-limit : 1 calls per 1 SECONDS period for this particular
         operation (polling)
         """
+        await self._refresh_token_if_expired()
         response = await self.__post(f"events/{self.event_listener_id}/fetch")
         events = [Event(**e) for e in humps.decamelize(response)]
 
@@ -529,6 +558,7 @@ class OverkizClient:
         Unregister an event listener.
         API response status is always 200, even on unknown listener ids.
         """
+        await self._refresh_token_if_expired()
         await self.__post(f"events/{self.event_listener_id}/unregister")
         self.event_listener_id = None
 
@@ -552,6 +582,15 @@ class OverkizClient:
 
         return executions
 
+    @backoff.on_exception(
+        backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
+    )
+    async def get_api_version(self) -> str:
+        """Get the API version (local only)"""
+        response = await self.__get("apiVersion")
+
+        return cast(str, response["protocolVersion"])
+
     @backoff.on_exception(backoff.expo, TooManyExecutionsException, max_tries=10)
     @backoff.on_exception(
         backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
@@ -617,6 +656,74 @@ class OverkizClient:
         places = Place(**humps.decamelize(response))
         return places
 
+    @backoff.on_exception(
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError),
+        max_tries=2,
+        on_backoff=relogin,
+    )
+    async def generate_local_token(self, gateway_id: str) -> str:
+        """
+        Generates a new token
+        Access scope : Full enduser API access (enduser/*)
+        """
+        response = await self.__get(f"config/{gateway_id}/local/tokens/generate")
+
+        return cast(str, response["token"])
+
+    @backoff.on_exception(
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError),
+        max_tries=2,
+        on_backoff=relogin,
+    )
+    async def activate_local_token(
+        self, gateway_id: str, token: str, label: str, scope: str = "devmode"
+    ) -> str:
+        """
+        Create a token
+        Access scope : Full enduser API access (enduser/*)
+        """
+        response = await self.__post(
+            f"config/{gateway_id}/local/tokens",
+            {"label": label, "token": token, "scope": scope},
+        )
+
+        return cast(str, response["requestId"])
+
+    @backoff.on_exception(
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError),
+        max_tries=2,
+        on_backoff=relogin,
+    )
+    async def get_local_tokens(
+        self, gateway_id: str, scope: str = "devmode"
+    ) -> list[LocalToken]:
+        """
+        Get all gateway tokens with the given scope
+        Access scope : Full enduser API access (enduser/*)
+        """
+        response = await self.__get(f"config/{gateway_id}/local/tokens/{scope}")
+        local_tokens = [LocalToken(**lt) for lt in humps.decamelize(response)]
+
+        return local_tokens
+
+    @backoff.on_exception(
+        backoff.expo,
+        (NotAuthenticatedException, ServerDisconnectedError),
+        max_tries=2,
+        on_backoff=relogin,
+    )
+    async def delete_local_token(self, gateway_id: str, uuid: str) -> bool:
+        """
+        Delete a token
+        Access scope : Full enduser API access (enduser/*)
+        """
+        await self.__delete(f"config/{gateway_id}/local/tokens/{uuid}")
+
+        return True
+
     @backoff.on_exception(
         backoff.expo, NotAuthenticatedException, max_tries=2, on_backoff=relogin
     )
@@ -629,12 +736,13 @@ class OverkizClient:
         """Make a GET request to the OverKiz API"""
         headers = {}
 
-        if self.server == SUPPORTED_SERVERS["somfy_europe"]:
-            await self._refresh_somfy_tahoma_token_if_expired()
+        await self._refresh_token_if_expired()
+        if self._access_token:
             headers["Authorization"] = f"Bearer {self._access_token}"
 
         async with self.session.get(
-            f"{self.server.endpoint}{path}", headers=headers
+            f"{self.server.endpoint}{path}",
+            headers=headers,
         ) as response:
             await self.check_response(response)
             return await response.json()
@@ -645,8 +753,8 @@ class OverkizClient:
         """Make a POST request to the OverKiz API"""
         headers = {}
 
-        if self.server == SUPPORTED_SERVERS["somfy_europe"] and path != "login":
-            await self._refresh_somfy_tahoma_token_if_expired()
+        if path != "login" and self._access_token:
+            await self._refresh_token_if_expired()
             headers["Authorization"] = f"Bearer {self._access_token}"
 
         async with self.session.post(
@@ -659,8 +767,9 @@ class OverkizClient:
         """Make a DELETE request to the OverKiz API"""
         headers = {}
 
-        if self.server == SUPPORTED_SERVERS["somfy_europe"]:
-            await self._refresh_somfy_tahoma_token_if_expired()
+        await self._refresh_token_if_expired()
+
+        if self._access_token:
             headers["Authorization"] = f"Bearer {self._access_token}"
 
         async with self.session.delete(
@@ -700,6 +809,10 @@ class OverkizClient:
             if message == "Not authenticated":
                 raise NotAuthenticatedException(message)
 
+            # {"error":"Missing authorization token.","errorCode":"RESOURCE_ACCESS_DENIED"}
+            if message == "Missing authorization token.":
+                raise MissingAuthorizationTokenException(message)
+
             # {"error": "Server busy, please try again later. (Too many executions)"}
             if message == "Server busy, please try again later. (Too many executions)":
                 raise TooManyExecutionsException(message)
@@ -716,16 +829,46 @@ class OverkizClient:
             if message == "No registered event listener":
                 raise NoRegisteredEventListenerException(message)
 
+            # {"errorCode": "RESOURCE_ACCESS_DENIED",  "error": "too many concurrent requests"}
+            if message == "too many concurrent requests":
+                raise TooManyConcurrentRequestsException(message)
+
+            if message == "Cannot use JSESSIONID and bearer token in same request":
+                raise SessionAndBearerInSameRequestException(message)
+
+            if (
+                message
+                == "Too many attempts with an invalid token, temporarily banned."
+            ):
+                raise TooManyAttemptsBannedException(message)
+
+            if "Invalid token : " in message:
+                raise InvalidTokenException(message)
+
+            if "Not such token with UUID: " in message:
+                raise NotSuchTokenException(message)
+
+            if "Unknown user :" in message:
+                raise UnknownUserException(message)
+
+            # {"error":"Unknown object.","errorCode":"UNSPECIFIED_ERROR"}
+            if message == "Unknown object.":
+                raise UnknownObjectException(message)
+
+            # {'errorCode': 'RESOURCE_ACCESS_DENIED', 'error': 'Access denied to gateway #1234-5678-1234 for action ADD_TOKEN'}
+            if "Access denied to gateway" in message:
+                raise AccessDeniedToGatewayException(message)
+
         raise Exception(message if message else result)
 
-    async def _refresh_somfy_tahoma_token_if_expired(self) -> None:
+    async def _refresh_token_if_expired(self) -> None:
         """Check if token is expired and request a new one."""
         if (
             self._expires_in
             and self._refresh_token
             and self._expires_in <= datetime.datetime.now()
         ):
-            await self.somfy_tahoma_refresh_token(self._refresh_token)
+            await self.refresh_token()
 
             if self.event_listener_id:
                 await self.register_event_listener()
