@@ -23,6 +23,24 @@ SUPPORTED_SERVERS: dict[str, OverkizServer] = {
         manufacturer="Atlantic",
         configuration_url=None,
     ),
+    "brandt": OverkizServer(
+        name="Brandt Smart Control",
+        endpoint="https://ha3-1.overkiz.com/enduser-mobile-web/enduserAPI/",
+        manufacturer="Brandt",
+        configuration_url=None,
+    ),
+    "flexom": OverkizServer(
+        name="Flexom",
+        endpoint="https://ha108-1.overkiz.com/enduser-mobile-web/enduserAPI/",
+        manufacturer="Bouygues",
+        configuration_url=None,
+    ),
+    "hexaom_hexaconnect": OverkizServer(
+        name="Hexaom HexaConnect",
+        endpoint="https://ha5-1.overkiz.com/enduser-mobile-web/enduserAPI/",
+        manufacturer="Hexaom",
+        configuration_url=None,
+    ),
     "hi_kumo_asia": OverkizServer(
         name="Hitachi Hi Kumo (Asia)",
         endpoint="https://ha203-1.overkiz.com/enduser-mobile-web/enduserAPI/",
@@ -53,9 +71,15 @@ SUPPORTED_SERVERS: dict[str, OverkizServer] = {
         manufacturer="Rexel",
         configuration_url="https://utilisateur.energeasyconnect.com/user/#/zone/equipements",
     ),
-    "somfy_europe": OverkizServer(  # uses https://ha101-1.overkiz.com
+    "simu_livein2": OverkizServer(  # alias of https://tahomalink.com
+        name="SIMU (LiveIn2)",
+        endpoint="https://ha101-1.overkiz.com/enduser-mobile-web/enduserAPI/",
+        manufacturer="Somfy",
+        configuration_url=None,
+    ),
+    "somfy_europe": OverkizServer(  # alias of https://tahomalink.com
         name="Somfy (Europe)",
-        endpoint="https://tahomalink.com/enduser-mobile-web/enduserAPI/",
+        endpoint="https://ha101-1.overkiz.com/enduser-mobile-web/enduserAPI/",
         manufacturer="Somfy",
         configuration_url="https://www.tahomalink.com",
     ),
@@ -71,4 +95,10 @@ SUPPORTED_SERVERS: dict[str, OverkizServer] = {
         manufacturer="Somfy",
         configuration_url=None,
     ),
+    "ubiwizz": OverkizServer(
+        name="Ubiwizz",
+        endpoint="https://ha129-1.overkiz.com/enduser-mobile-web/enduserAPI/",
+        manufacturer="Decelect",
+        configuration_url=None,
+    ),
 }
