@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from typing import Any, Iterator
+from typing import Any, Iterator, cast
 
 from attr import define, field
 
@@ -27,51 +27,51 @@ from pyoverkiz.types import DATA_TYPE_TO_PYTHON, StateType
 
 @define(init=False, kw_only=True)
 class Setup:
-    creation_time: str
+    creation_time: str | None = None
     last_update_time: str | None = None
     id: str = field(repr=obfuscate_id, default=None)
-    location: Location
+    location: Location | None = None
     gateways: list[Gateway]
     devices: list[Device]
-    zones: list[Zone]
-    reseller_delegation_type: str
-    oid: str
-    root_place: Place
+    zones: list[Zone] | None = None
+    reseller_delegation_type: str | None = None
+    oid: str | None = None
+    root_place: Place | None = None
     features: list[Feature] | None = None
 
     def __init__(
         self,
         *,
-        creation_time: str,
+        creation_time: str | None = None,
         last_update_time: str | None = None,
         id: str = field(repr=obfuscate_id, default=None),
-        location: dict[str, Any],
+        location: dict[str, Any] | None = None,
         gateways: list[dict[str, Any]],
         devices: list[dict[str, Any]],
-        zones: list[dict[str, Any]],
-        reseller_delegation_type: str,
-        oid: str,
-        root_place: dict[str, Any],
+        zones: list[dict[str, Any]] | None = None,
+        reseller_delegation_type: str | None = None,
+        oid: str | None = None,
+        root_place: dict[str, Any] | None = None,
         features: list[dict[str, Any]] | None = None,
         **_: Any,
     ) -> None:
         self.id = id
         self.creation_time = creation_time
         self.last_update_time = last_update_time
-        self.location = Location(**location)
+        self.location = Location(**location) if location else None
         self.gateways = [Gateway(**g) for g in gateways]
         self.devices = [Device(**d) for d in devices]
-        self.zones = [Zone(**z) for z in zones]
+        self.zones = [Zone(**z) for z in zones] if zones else None
         self.reseller_delegation_type = reseller_delegation_type
         self.oid = oid
-        self.root_place = Place(**root_place)
+        self.root_place = Place(**root_place) if root_place else None
         self.features = [Feature(**f) for f in features] if features else None
 
 
 @define(init=False, kw_only=True)
 class Location:
     creation_time: str
-    last_update_time: str
+    last_update_time: str | None = None
     city: str = field(repr=obfuscate_string, default=None)
     country: str = field(repr=obfuscate_string, default=None)
     postal_code: str = field(repr=obfuscate_string, default=None)
@@ -93,7 +93,7 @@ class Location:
         self,
         *,
         creation_time: str,
-        last_update_time: str,
+        last_update_time: str | None = None,
         city: str = field(repr=obfuscate_string, default=None),
         country: str = field(repr=obfuscate_string, default=None),
         postal_code: str = field(repr=obfuscate_string, default=None),
@@ -147,7 +147,7 @@ class Device:
     ui_class: UIClass
     states: States
     type: ProductType
-    place_oid: str
+    place_oid: str | None = None
     protocol: Protocol = field(init=False, repr=False)
 
     def __init__(
@@ -161,11 +161,12 @@ class Device:
         controllable_name: str,
         definition: dict[str, Any],
         data_properties: list[dict[str, Any]] | None = None,
-        widget: str,
-        ui_class: str,
+        widget: str | None = None,
+        widget_name: str | None = None,
+        ui_class: str | None = None,
         states: list[dict[str, Any]] | None = None,
         type: int,
-        place_oid: str,
+        place_oid: str | None = None,
         **_: Any,
     ) -> None:
         self.id = device_url
@@ -178,27 +179,44 @@ class Device:
         self.controllable_name = controllable_name
         self.states = States(states)
         self.data_properties = data_properties
-        self.widget = UIWidget(widget)
-        self.ui_class = UIClass(ui_class)
         self.type = ProductType(type)
         self.place_oid = place_oid
         self.protocol = Protocol(self.device_url.split(":")[0])
 
+        if ui_class:
+            self.ui_class = UIClass(ui_class)
+        elif self.definition.ui_class:
+            self.ui_class = UIClass(self.definition.ui_class)
+
+        if widget:
+            self.widget = UIWidget(widget)
+        elif self.definition.widget_name:
+            self.widget = UIWidget(self.definition.widget_name)
+
 
 @define(init=False, kw_only=True)
 class StateDefinition:
 
     qualified_name: str
-    type: str
+    type: str | None = None
     values: list[str] | None = None
 
     def __init__(
-        self, qualified_name: str, type: str, values: list[str] | None = None, **_: Any
+        self,
+        name: str | None = None,
+        qualified_name: str | None = None,
+        type: str | None = None,
+        values: list[str] | None = None,
+        **_: Any,
     ) -> None:
-        self.qualified_name = qualified_name
         self.type = type
         self.values = values
 
+        if qualified_name:
+            self.qualified_name = qualified_name
+        elif name:
+            self.qualified_name = name
+
 
 @define(init=False, kw_only=True)
 class Definition:
@@ -206,7 +224,7 @@ class Definition:
     states: list[StateDefinition]
     widget_name: str | None = None
     ui_class: str | None = None
-    qualified_name: str
+    qualified_name: str | None = None
 
     def __init__(
         self,
@@ -215,7 +233,7 @@ class Definition:
         states: list[dict[str, Any]] | None = None,
         widget_name: str | None = None,
         ui_class: str | None = None,
-        qualified_name: str,
+        qualified_name: str | None = None,
         **_: Any,
     ) -> None:
         self.commands = CommandDefinitions(commands)
@@ -274,6 +292,54 @@ class State:
         self.value = value
         self.type = DataType(type)
 
+    @property
+    def value_as_int(self) -> int | None:
+        if self.type == DataType.NONE:
+            return None
+        if self.type == DataType.INTEGER:
+            return cast(int, self.value)
+        raise TypeError(f"{self.name} is not an integer")
+
+    @property
+    def value_as_float(self) -> float | None:
+        if self.type == DataType.NONE:
+            return None
+        if self.type == DataType.FLOAT:
+            return cast(float, self.value)
+        raise TypeError(f"{self.name} is not a float")
+
+    @property
+    def value_as_bool(self) -> bool | None:
+        if self.type == DataType.NONE:
+            return None
+        if self.type == DataType.BOOLEAN:
+            return cast(bool, self.value)
+        raise TypeError(f"{self.name} is not a boolean")
+
+    @property
+    def value_as_str(self) -> str | None:
+        if self.type == DataType.NONE:
+            return None
+        if self.type == DataType.STRING:
+            return cast(str, self.value)
+        raise TypeError(f"{self.name} is not a string")
+
+    @property
+    def value_as_dict(self) -> dict[str, Any] | None:
+        if self.type == DataType.NONE:
+            return None
+        if self.type == DataType.JSON_OBJECT:
+            return cast(dict, self.value)
+        raise TypeError(f"{self.name} is not a JSON object")
+
+    @property
+    def value_as_list(self) -> list[Any] | None:
+        if self.type == DataType.NONE:
+            return None
+        if self.type == DataType.JSON_ARRAY:
+            return cast(list, self.value)
+        raise TypeError(f"{self.name} is not an array")
+
 
 @define(init=False, kw_only=True)
 class EventState(State):
@@ -286,9 +352,10 @@ class EventState(State):
     ):
         super().__init__(name, type, value, **_)
 
-        # Overkiz returns all state values as a string
+        # Overkiz (cloud) returns all state values as a string
         # We cast them here based on the data type provided by Overkiz
-        if self.type in DATA_TYPE_TO_PYTHON:
+        # Overkiz (local) returns all state values in the right format
+        if isinstance(self.value, str) and self.type in DATA_TYPE_TO_PYTHON:
             self.value = DATA_TYPE_TO_PYTHON[self.type](self.value)
 
 
@@ -340,8 +407,8 @@ class Command(dict):
 
 @define(init=False, kw_only=True)
 class Event:
-    timestamp: int
     name: EventName
+    timestamp: int | None
     setupoid: str | None = field(repr=obfuscate_id, default=None)
     owner_key: str | None = field(repr=obfuscate_id, default=None)
     type: int | None = None
@@ -366,8 +433,8 @@ class Event:
 
     def __init__(
         self,
-        timestamp: int,
         name: EventName,
+        timestamp: int | None = None,
         setupoid: str | None = field(repr=obfuscate_id, default=None),
         owner_key: str | None = None,
         type: int | None = None,
@@ -487,14 +554,14 @@ class Gateway:
     id: str = field(repr=obfuscate_id)
     gateway_id: str = field(repr=obfuscate_id)
     alive: bool | None = None
-    mode: str
+    mode: str | None = None
     place_oid: str | None = None
-    time_reliable: bool
+    time_reliable: bool | None = None
     connectivity: Connectivity
     up_to_date: bool | None = None
-    update_status: UpdateBoxStatus
-    sync_in_progress: bool
-    type: GatewayType
+    update_status: UpdateBoxStatus | None = None
+    sync_in_progress: bool | None = None
+    type: GatewayType | None = None
 
     def __init__(
         self,
@@ -504,14 +571,14 @@ class Gateway:
         sub_type: GatewaySubType | None = None,
         gateway_id: str,
         alive: bool | None = None,
-        mode: str,
+        mode: str | None = None,
         place_oid: str | None = None,
-        time_reliable: bool,
+        time_reliable: bool | None = None,
         connectivity: dict[str, Any],
         up_to_date: bool | None = None,
-        update_status: UpdateBoxStatus,
-        sync_in_progress: bool,
-        type: GatewayType,
+        update_status: UpdateBoxStatus | None = None,
+        sync_in_progress: bool | None = None,
+        type: GatewayType | None = None,
         **_: Any,
     ) -> None:
         self.id = gateway_id
@@ -523,10 +590,10 @@ class Gateway:
         self.time_reliable = time_reliable
         self.connectivity = Connectivity(**connectivity)
         self.up_to_date = up_to_date
-        self.update_status = UpdateBoxStatus(update_status)
+        self.update_status = UpdateBoxStatus(update_status) if update_status else None
         self.sync_in_progress = sync_in_progress
         self.partners = [Partner(**p) for p in partners] if partners else []
-        self.type = GatewayType(type)
+        self.type = GatewayType(type) if type else None
         self.sub_type = GatewaySubType(sub_type) if sub_type else None
 
 
@@ -662,8 +729,8 @@ class Zone:
     label: str
     type: int
     items: list[ZoneItem] | None
-    external_oid: str
-    metadata: str
+    external_oid: str | None
+    metadata: str | None
     oid: str
 
     def __init__(
@@ -673,8 +740,8 @@ class Zone:
         label: str,
         type: int,
         items: list[dict[str, Any]] | None,
-        external_oid: str,
-        metadata: str,
+        external_oid: str | None = None,
+        metadata: str | None = None,
         oid: str,
         **_: Any,
     ) -> None:
@@ -695,3 +762,13 @@ class OverkizServer:
     endpoint: str
     manufacturer: str
     configuration_url: str | None
+
+
+@define(kw_only=True)
+class LocalToken:
+    label: str
+    gateway_id: str = field(repr=obfuscate_id, default=None)
+    expiration_time: int
+    gateway_creation_time: int
+    uuid: str
+    scope: str
